/**
 * @link https://www.shopwind.net/
 * @copyright Copyright (c) 2018 ShopWind Inc. All Rights Reserved.
 *
 * This is not free software. Do not use it for commercial purposes. 
 * If you need commercial operation, please contact us to purchase a license.
 * @license https://www.shopwind.net/license/
 * ==========================================================================
 * 
 * @Id pay.js 2019.10.22 $
 * @author winder
 */

import server from '@/common/server.js'
import util from '@/common/util.js'

/**
 * 获取收银台数据
 * @param {Object} that
 */
function cashier(that) {
	server.request('cashier/build', {
		bizOrderId: that.bizOrderId,
		// #ifdef APP-PLUS
		terminal: 'APP',
		// #endif
		// #ifdef MP
		terminal: 'MP',
		// #endif
		// #ifdef H5
		terminal: 'WAP',
		// #endif
		bizIdentity: that.bizIdentity
	}, function(res) {
		if (res.code == 0) {
			that.orderInfo = res.data.orderInfo
			that.payments = res.data.payments
			that.orderId = res.data.orderId

			// 绑定默认支付方式
			defpayment(that)
		} else {
			uni.showModal({
				title: '提示',
				content: res.message,
				showCancel: false,
				success() {
					uni.redirectTo({
						url: '/pages/my/order/list'
					})
				}
			})
		}
		uni.stopPullDownRefresh()
	})
}

/**
 * 绑定默认支付方式
 * @param {Object} that
 */
function defpayment(that) {
	for (let index in that.payments) {
		if (that.payments[index].selected && !that.payments[index].disabled) {
			that.payment_code = index
			break
		}
		if (!that.payments[index].disabled && !that.payment_code) {
			that.payment_code = index
		}
	}
}

/**
 * 提交支付
 * @param {Object} that
 * @param {Object} password
 */
async function submit(that, password) {
	if (that.paysubmit) {
		uni.showToast({
			title: "请不要重复提交",
			icon: 'loading',
			mask: true
		})
		return
	}

	if (that.payment_code == '') {
		uni.showModal({
			title: '提示',
			content: "请选择付款方式",
			showCancel: false
		})
	} else {

		// 标记不允许重复提交
		that.paysubmit = true;
		setTimeout(function() {
			that.paysubmit = false
		}, 5000)

		// 微信小程序支付才需要
		let code = await util.getWxCode()
		server.request('cashier/pay', {
			// #ifdef H5
			callback: util.payCallback(that.payment_code),
			terminal: 'WAP',
			// #endif
			// #ifdef APP-PLUS
			terminal: 'APP',
			// #endif
			// #ifdef MP
			terminal: 'MP',
			// #endif
			// #ifdef MP-WEIXIN
			code: code,
			// #endif
			orderId: that.orderId,
			payment_code: that.payment_code,
			password: password
		}, function(res) {
			if (res.code == 0) {

				// 微信公众号支付/H5支付(跳转)
				if (res.data.redirect) {
					location.href = res.data.redirect
				}

				// 货到付款 & 余额支付
				else if (that.payment_code == 'cod' || that.payment_code == 'deposit') {
					uni.redirectTo({
						url: '/pages/trade/cashier/result?payTradeNo=' + res.data.payTradeNo
					})
				} else {
					uni.requestPayment({
						provider: that.payment_code,
						// #ifdef APP-PLUS
						orderInfo: res.data.orderInfo, // APP端微信(object)、支付宝(string)订单数据
						// #endif

						// #ifdef MP-WEIXIN
						timeStamp: res.data.orderInfo.timeStamp,
						nonceStr: res.data.orderInfo.nonceStr,
						package: res.data.orderInfo.package,
						signType: res.data.orderInfo.signType,
						paySign: res.data.orderInfo.paySign,
						// #endif

						// #ifdef MP-TOUTIAO
						service: 4,
						_debug: 1,
						// #endif

						success: function(suc) {
							uni.redirectTo({
								url: '/pages/trade/cashier/result?payTradeNo=' + res.data
									.payTradeNo
							})
						},
						fail: function(err) {
							let content = '支付失败'
							if (err.errMsg.indexOf('requestPayment:fail') > -1) {
								if (err.errMsg.indexOf('cancel') > -1) {
									content = '已取消支付'
								} else {
									content = err.errMsg
								}
							}
							uni.showModal({
								title: '提示',
								content: content,
								showCancel: false
							})
						}
					})
				}
			} else {
				uni.showModal({
					title: '提示',
					content: res.message,
					showCancel: false
				})
			}
		}, true)
	}
}

/**
 * 获取微信公众号支付
 * 购物订单/充值订单都会跳转到该页面
 * @param {Object} that
 * @param {Object} params
 */
function wxpaybuild(that, params) {
	server.request('cashier/wxpay', params, function(res) {
		if (res.code == 0) {
			that.orderInfo = res.data.orderInfo
			if (!res.data.jsApiParameters) {
				uni.showModal({
					title: '提示',
					content: '支付参数有误，无法完成支付！',
					showCancel: false,
					success() {
						util.pageBack(2)
					}
				})
				return
			}
			that.jsApiParameters = res.data.jsApiParameters
		} else if (res.message) {
			uni.showToast({
				title: res.message,
				icon: 'error'
			})
		}
	}, true)
}

/**
 * 发起微信公众号JSAPI支付
 * 购物订单/充值订单都会回调到该页面处理
 * @param {Object} that
 */
function wxpay(that) {
	if (that.paysubmit) {
		uni.showToast({
			title: "请不要重复提交",
			icon: 'loading',
			mask: true
		})
		return
	}
	if (!util.isWechat()) {
		uni.showToast({
			title: '请在微信中支付',
			icon: 'none',
			mask: true
		})
	}

	// 标记不允许重复提交
	that.paysubmit = true;
	setTimeout(function() {
		that.paysubmit = false
	}, 5000)

	var wx = require('jweixin-module')
	wx.config({
		debug: false, // 开启调试模式,调用的所有api的返回值会在客户端alert出来，若要查看传入的参数，可以在pc端打开，参数信息会通过log打出，仅在pc端时才会打印。
		appId: that.jsApiParameters.appId, // 必填，公众号的唯一标识
		timestamp: that.jsApiParameters.timeStamp, // 必填，生成签名的时间戳
		nonceStr: that.jsApiParameters.nonceStr, // 必填，生成签名的随机串
		signature: that.jsApiParameters.paySign, // 必填，签名
		jsApiList: ['chooseWXPay'] // 必填，需要使用的JS接口列表
	});
	wx.ready(function() {
		wx.chooseWXPay({
			timestamp: that.jsApiParameters
				.timeStamp, // 支付签名时间戳，注意微信jssdk中的所有使用timestamp字段均为小写。但最新版的支付后台生成签名使用的timeStamp字段名需大写其中的S字符
			nonceStr: that.jsApiParameters.nonceStr, // 支付签名随机串，不长于 32 位
			package: that.jsApiParameters.package, // 统一支付接口返回的prepay_id参数值，提交格式如：prepay_id=\*\*\*）
			signType: that.jsApiParameters.signType, // 微信支付V3的传入RSA,微信支付V2的传入格式与V2统一下单的签名格式保持一致
			paySign: that.jsApiParameters.paySign, // 支付签名
			success: function(res) {
				uni.redirectTo({
					url: '/pages/trade/cashier/result?payTradeNo=' + that.orderInfo.payTradeNo
				})
			}
		})
	})
}

module.exports = {
	cashier: cashier,
	submit: submit,
	wxpay: wxpay,
	wxpaybuild: wxpaybuild
}