/**
 * @link https://www.shopwind.net/
 * @copyright Copyright (c) 2018 ShopWind Inc. All Rights Reserved.
 *
 * This is not free software. Do not use it for commercial purposes. 
 * If you need commercial operation, please contact us to purchase a license.
 * @license https://www.shopwind.net/license/
 * ==========================================================================
 * 
 * @Id recharge.js 2019.11.4 $
 * @author winder
 */

import server from '@/common/server.js'
import util from '@/common/util.js'

/**
 * 提交充值请求
 * @param {Object} that
 */
async function submit(that) {
	if (that.paysubmit) {
		uni.showToast({
			title: "请不要重复提交",
			icon: 'loading',
			mask: true
		})
		return
	}

	// 标记不允许重复提交
	that.paysubmit = true;
	setTimeout(function() {
		that.paysubmit = false
	}, 5000)

	// 微信小程序支付才需要
	let code = await util.getWxCode()
	server.request('deposit/recharge', {
		// #ifdef H5
		callback: util.payCallback(that.payment.code),
		terminal: 'WAP',
		// #endif
		// #ifdef APP-PLUS
		terminal: 'APP',
		//#endif
		// #ifdef MP
		terminal: 'MP',
		// #endif
		// #ifdef MP-WEIXIN
		code: code,
		// #endif
		money: that.money,
		payment_code: that.payment.code
	}, function(res) {
		if (res.code == 0) {
			// 微信公众号支付/H5支付(跳转)
			if (res.data.redirect) {
				location.href = res.data.redirect
			} else {
				uni.requestPayment({
					provider: that.payment.code,
					// #ifdef APP-PLUS
					orderInfo: res.data.orderInfo, // APP端微信(object)、支付宝(string)订单数据
					// #endif

					// #ifdef MP-WEIXIN
					timeStamp: res.data.orderInfo.timeStamp,
					nonceStr: res.data.orderInfo.nonceStr,
					package: res.data.orderInfo.package,
					signType: res.data.orderInfo.signType,
					paySign: res.data.orderInfo.paySign,
					// #endif

					success: function(suc) {
						uni.redirectTo({
							url: '/pages/trade/cashier/result?payTradeNo=' + res.data
								.payTradeNo
						})
					},
					fail: function(err) {
						let content = '充值失败'
						if (err.errMsg.indexOf('requestPayment:fail') > -1) {
							if (err.errMsg.indexOf('cancel') > -1) {
								content = '已取消充值'
							} else {
								content = err.errMsg
							}
						}
						uni.showModal({
							title: '提示',
							content: content,
							showCancel: false
						})
					}
				})
			}
		} else {
			uni.showToast({
				title: res.message,
				icon: "none"
			})
		}
	})
}

/**
 * 获取可用的充值方式
 * 后期建议从接口获取
 * @param {Object} that
 */
function getPayments(that) {

	// #ifdef H5
	that.payments = [{
		code: 'wxpay',
		name: '微信支付'
	}]
	if (!util.isWechat()) {
		that.payments.push({
			code: 'alipay',
			name: '支付宝'
		})
	}
	// #endif

	// #ifdef APP-PLUS
	that.payments = [{
			code: 'wxpay',
			name: '微信支付'
		},
		{
			code: 'alipay',
			name: '支付宝'
		}
	]
	// #endif

	// #ifdef MP-WEIXIN
	that.payments = [{
		code: 'wxpay',
		name: '微信支付'
	}]
	// #endif

	that.payment = that.payments[0]
}

module.exports = {
	submit: submit,
	getPayments: getPayments
}